package eu.clarin.cmdi.oai.harvester;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.List;

import javax.xml.stream.XMLStreamReader;


public interface HarvestHandler {

    /**
     * Receive notification after harvester has identifyed the repository.
     * 
     * @param repository
     *            information about the repository
     * @see Repository
     */
    public void onIdentify(Repository repository);

    /**
     * Receive notification after enumerating all supported metadata formats
     * from a repository.
     * 
     * @param metadataFormats
     *            list of supported metadata formats
     * @see MetadataFormat
     */
    public void onListMetadataFormats(List<MetadataFormat> metadataFormats);

    /**
     * Receive notification before  arvesting a metadata prefix.
     * 
     * @param prefix
     *            metadata prefix to be harvested
     */
    public void onStartListingRecords(String prefix);

    /**
     * Receive notification after harvesting a metadata prefix.
     * 
     * @param prefix
     *            metadata prefix which was harvested
     * @param latestDatestamp
     *            the datestamp of the latest record for this prefix
     */
    public void onFinishListingRecords(String prefix, Date latestDatestamp);

    /**
     * Receive notification of a metadata record.
     * 
     * @param header
     *            the header for the new metadata record. This object is only
     *            valid for invocation of this method.
     * @see Header
     */
    public void onRecord(Header header);

    /**
     * Receive notification of the content of a metadata record.
     * 
     * @param header
     *            the header for the new metadata record. This object is only
     *            valid for invocation of this method.
     * @param reader
     *            a reader for extracting the content
     * @see Header
     * @see javax.xml.stream.XMLStreamReader
     */
    public void onRecordMetadata(Header header, XMLStreamReader reader);

    /**
     * Receive notification of the additional information about a metadata
     * record.
     * 
     * @param header
     *            the header for the new metadata record. This object is only
     *            valid for invocation of this method.
     * @param reader
     *            a reader for extracting the additional information
     * @see Header
     * @see javax.xml.stream.XMLStreamReader
     */
    public void onRecordAbout(Header header, XMLStreamReader reader);

    /* XXX: move to different interface? */
    public InputStream wrap(InputStream stream) throws IOException;
    
} // interface HarvestHandler
