package eu.clarin.cmdi.oai.harvester;
import java.util.Date;
import java.util.List;

/**
 * Interface for accessing the headers from harvested records.
 */
public interface Header {

    /**
     * Get the identifier of the record.
     * 
     * @return the record identifier
     */
    public String getIdentifier();
    
    /**
     * Get the date stamp of the record. The date stamp will automatically be
     * converted from UTC to the local time zone.
     * 
     * @return the date stamp of the record
     */
    public Date getDatestamp();
    
    /**
     * Get the set names the record is a member of.
     * 
     * @return the a list set names s or <code>null</code> 
     */
    public List<String> getSets();
    
    /**
     * Get the deleted state of the record
     * 
     * @return <code>true</code> if the record is deleted,
     *         <code>false</code> otherwise
     */
    public boolean isDeleted();
    
} // class Header
