package eu.clarin.cmdi.oai.harvester.impl;

import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.codehaus.stax2.XMLInputFactory2;
import org.codehaus.stax2.XMLStreamReader2;

import eu.clarin.cmdi.oai.harvester.HarvestHandler;
import eu.clarin.cmdi.oai.harvester.HarvestJob;
import eu.clarin.cmdi.oai.harvester.Harvester;
import eu.clarin.cmdi.oai.harvester.HarvesterException;

abstract class AbstractHarvester implements Harvester {
    private final XMLInputFactory factory;
//    private final XMLValidationSchema schema;
    
    
    protected AbstractHarvester() {
        factory = XMLInputFactory2.newInstance();
        factory.setProperty(XMLInputFactory.IS_COALESCING, Boolean.TRUE);
        factory.setProperty(XMLInputFactory.IS_NAMESPACE_AWARE,
                Boolean.TRUE);
        factory.setProperty(XMLInputFactory2.P_AUTO_CLOSE_INPUT,
                Boolean.FALSE);
        // XMLValidationSchemaFactory schemaFactory =
        // XMLValidationSchemaFactory
        // .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        // URL url = Harvester.class.getClassLoader().getResource(
        // "META-INF/OAI-PMH.xsd");
        // if (url == null) {
        // throw new NullPointerException("schema not found!");
        // }
        // schema = schemaFactory.createSchema(url);
    }

    public abstract HarvestJob createJob(URI repositoryURI,
            HarvestHandler handler) throws HarvesterException;

    public final HarvestJob createJob(URI repositoryURI)
            throws HarvesterException {
        return createJob(repositoryURI, null);
    }
    
    @Override
    public final HarvestJob createJob(String repositoryURI,
            HarvestHandler handler) throws HarvesterException {
        try {
            return createJob(new URI(repositoryURI), handler);
        } catch (URISyntaxException e) {
            throw new HarvesterException("invalid repositoryURI", e);
        }
    }

    @Override
    public final HarvestJob createJob(String repositoryURI)
            throws HarvesterException {
        return createJob(repositoryURI, null);
    }

    @Override
    public final HarvestJob findJob(String id) throws HarvesterException {
        if (id == null) {
            throw new NullPointerException("id == null");
        }
        return doFindJob(id);
    }

    @Override
    public final HarvestJob cancelJob(String id) throws HarvesterException {
        if (id == null) {
            throw new NullPointerException("id == null");
        }
        HarvestJob job = doFindJob(id);
        if (job != null) {
            job.cancel();
        }
        return job;
    }

    abstract void doRunJob(HarvestJobImpl job) throws HarvesterException;

    abstract void doCancelJob(HarvestJobImpl job) throws HarvesterException;

    XMLStreamReader createReader(InputStream in) throws XMLStreamException {
        XMLStreamReader2 reader =
            (XMLStreamReader2) factory.createXMLStreamReader(in);
        // reader.validateAgainst(schema);
        return reader;
    }

    protected abstract HarvestJob doFindJob(String id)
        throws HarvesterException;

} // abstract class AbstractHarvester
