package eu.clarin.cmdi.oai.harvester.impl;

import java.net.URI;
import java.net.URISyntaxException;

final class RequestUriBuilder {
    private static final String SEPARATOR_PARAMETER = "&";
    private static final String SEPARATOR_NAME_VALUE = "=";
    private static final String ARG_VERB = "verb";
    private final StringBuilder builder = new StringBuilder();


    RequestUriBuilder setVerb(String verb) {
        builder.setLength(0); // clear builder
        return addParameter(ARG_VERB, verb);
    }

    public RequestUriBuilder addParameter(String name, String value) {
        if (builder.length() > 0) {
            builder.append(SEPARATOR_PARAMETER);
        }
        builder.append(name);
        builder.append(SEPARATOR_NAME_VALUE);
        builder.append(value);
        return this;
    }

    public URI createURI(URI baseURI) {
        try {
            return new URI(baseURI.getScheme(), null, baseURI.getHost(),
                    baseURI.getPort(), baseURI.getPath(), builder.toString(),
                    null);
        } catch (URISyntaxException e) {
            throw new InternalError("should not happen: " + e);
        }
    }

} // class RequestUriBuilder
