package eu.clarin.cmdi.oai.harvester.impl;

import java.net.URI;

import eu.clarin.cmdi.oai.harvester.HarvestHandler;
import eu.clarin.cmdi.oai.harvester.HarvestJob;
import eu.clarin.cmdi.oai.harvester.Harvester;
import eu.clarin.cmdi.oai.harvester.HarvesterException;

public final class SimpleHarvester extends AbstractHarvester {
    private HarvestWorker worker;
    private long nextId = 0;

    private SimpleHarvester() {
        this.worker = new HarvestWorker(this);
    }

    @Override
    public HarvestJob createJob(URI repositoryURI, HarvestHandler handler) {
        if (repositoryURI == null) {
            throw new IllegalArgumentException("repositoryURI == null");
        }
        String jobId;
        synchronized (this) {
            jobId = "0x" + Long.toHexString(++nextId);
        } // synchronized
        return new HarvestJobImpl(this, jobId, repositoryURI, handler);
    }

    public static Harvester newInstance() {
        return new SimpleHarvester();
    }

    @Override
    void doRunJob(HarvestJobImpl job) throws HarvesterException {
        if (job.getState() != HarvestJob.State.NEW) {
            throw new IllegalStateException("job is already in state: " +
                    job.getState());
        }

        job.setState(HarvestJob.State.RUNNING);
        boolean done = false;
        while (!done) {
            done = job.process(worker);
        } // while
    }

    @Override
    void doCancelJob(HarvestJobImpl job) throws HarvesterException {
        throw new HarvesterException("cancel() is not supported");
    }

    @Override
    protected HarvestJob doFindJob(String id) throws HarvesterException {
        throw new HarvesterException("findJob() is not supported"); 
    }

} // class SimpleHarvester
