package eu.clarin.cmdi.oai.harvester.util;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

public final class XMLStreamCopier {

    public static void copy(XMLStreamReader from, XMLStreamWriter to)
            throws XMLStreamException {
        while (from.hasNext()) {
            copyEvent(from, to);
            from.next();
        }
        copyEvent(from, to);
    }

    private static void copyEvent(XMLStreamReader from, XMLStreamWriter to)
            throws XMLStreamException {
        switch (from.getEventType()) {
        case XMLStreamConstants.START_DOCUMENT:
            {
                String version = from.getVersion();
                if (version == null || version.length() == 0) {
                    to.writeStartDocument();
                } else {
                    to.writeStartDocument(from.getCharacterEncodingScheme(),
                            from.getVersion());
                }
                to.writeCharacters("\n");
            }
            return;

        case XMLStreamConstants.END_DOCUMENT:
            to.writeCharacters("\n");
            to.writeEndDocument();
            return;

        case XMLStreamConstants.START_ELEMENT:
            copyStartElement(from, to);
            return;

        case XMLStreamConstants.END_ELEMENT:
            to.writeEndElement();
            return;

        case XMLStreamConstants.SPACE:
            to.writeCharacters(from.getTextCharacters(), from.getTextStart(),
                    from.getTextLength());
            return;

        case XMLStreamConstants.CDATA:
            to.writeCData(from.getText());
            return;

        case XMLStreamConstants.CHARACTERS:
            to.writeCharacters(from.getTextCharacters(), from.getTextStart(),
                    from.getTextLength());
            return;

        case XMLStreamConstants.COMMENT:
            to.writeComment(from.getText());
            return;

        case XMLStreamConstants.PROCESSING_INSTRUCTION:
            to.writeProcessingInstruction(from.getPITarget(), from.getPIData());
            return;

        case XMLStreamConstants.DTD:
        case XMLStreamConstants.ENTITY_REFERENCE:
        case XMLStreamConstants.ATTRIBUTE:
        case XMLStreamConstants.NAMESPACE:
        case XMLStreamConstants.ENTITY_DECLARATION:
        case XMLStreamConstants.NOTATION_DECLARATION:
            /* FALL_TROUGH */
        }
        throw new XMLStreamException("unsupported event type: " +
                from.getEventType());
    }

    private static void copyStartElement(XMLStreamReader from,
            XMLStreamWriter to) throws XMLStreamException {
        int nsCount = from.getNamespaceCount();
        if (nsCount > 0) { // yup, got some...
            for (int i = 0; i < nsCount; ++i) {
                String prefix = from.getNamespacePrefix(i);
                String uri = from.getNamespaceURI(i);
                if (prefix == null || prefix.length() == 0) { // default NS
                    to.setDefaultNamespace(uri);
                } else {
                    to.setPrefix(prefix, uri);
                }
            }
        }
        to.writeStartElement(from.getPrefix(), from.getLocalName(),
                from.getNamespaceURI());

        if (nsCount > 0) {
            // write namespace declarations
            for (int i = 0; i < nsCount; ++i) {
                String prefix = from.getNamespacePrefix(i);
                String uri = from.getNamespaceURI(i);

                if (prefix == null || prefix.length() == 0) { // default NS
                    to.writeDefaultNamespace(uri);
                } else {
                    to.writeNamespace(prefix, uri);
                }
            }
        }
        int attrCount = from.getAttributeCount();
        if (attrCount > 0) {
            for (int i = 0; i < attrCount; ++i) {
                to.writeAttribute(from.getAttributePrefix(i),
                        from.getAttributeNamespace(i),
                        from.getAttributeLocalName(i),
                        from.getAttributeValue(i));
            }
        }
    }

} // class XMLStreamCopier
